/*
Pet assistant with IOT

Pet assistent is an hardware-software solution for pet caring
There are 4 things the pet care can do:
1) Notify you if your dog is in an enviromen that is too hot/cold
2) Notify you when your dog is barking and making too much noise
3) Let you know without going inside the app (you can check the app as well) if it's hot/cold for a walk and what the humidity is
4) find your dog if you lost it.

Created By:
  Omer Ben-Artzi 205682164
  Sapir Peretz 204456164
  Shelly tsirulnik 207128190
*/
// Incudes
#define BLYNK_PRINT SerialUSB
// #define BLYNK_DEBUG SerialUSB
#include <ESP8266_Lib.h>
#include <BlynkSimpleShieldEsp8266.h>
#include <Adafruit_CircuitPlayground.h>


// defines
#define EspSerial Serial1
#define ESP8266_BAUD 9600

// user defines
#define MIN_TEMPERATURE 6
#define MAX_TEMPERATURE 28
#define FIVE_MINUTES (1000 * 60 * 5)
#define TEN_SECONDS (1000 * 10)
#define BARK_MESSAGE "Your dog is barking!"
#define TOO_HOT_MESSAGE "The weather is too hot! Take your dog back home!"
#define TOO_COLD_MESSAGE "The weather is too cold! Take your dog back home!"

// Globals
char auth[] = "XXXXXXXXXXXXX";  //Blynk AUTH TOKEN HERE
char ssid[] = "XXXXXXXXXXXXX";                    //wifi network name here
char pass[] = "XXXXXXXXXXXXX";                        //wifi password here
ESP8266 wifi(&EspSerial);
bool isFindMyDog = false;
double humidityLevel = 0;

int tempTimestamp = 0;
int barkTimestamp = 0;

// Functions
void setup() {
  CircuitPlayground.begin();
  SerialUSB.begin(9600);
  EspSerial.begin(ESP8266_BAUD);
  delay(10);
  Blynk.begin(auth, wifi, ssid, pass);

  tempTimestamp = millis();
  barkTimestamp = millis();
}


void handleBarking() {
  int value = CircuitPlayground.mic.soundPressureLevel(10);

  if (value >= 100 && millis() - barkTimestamp > TEN_SECONDS ) {
    Blynk.virtualWrite(V2, BARK_MESSAGE); 
  } 
}


void handleTemerature() {
  int curTemperature = CircuitPlayground.temperature();
  String msg = "";
  // Normlize beween 0 to 5
  int numberOfCircules = (((float)curTemperature - MIN_TEMPERATURE) / (MAX_TEMPERATURE-MIN_TEMPERATURE)) * 5;

  Blynk.virtualWrite(V5, curTemperature);

  // check min and max values
  if(curTemperature < MIN_TEMPERATURE){
    numberOfCircules = 1;
    msg = TOO_COLD_MESSAGE;
    
  } else if(curTemperature > MAX_TEMPERATURE){
    numberOfCircules = 5;
    msg = TOO_HOT_MESSAGE;
    
  }

  // send every 5 minutes.
  if (msg != "" && millis() - tempTimestamp > FIVE_MINUTES) {
    tempTimestamp = millis();
    Blynk.virtualWrite(V3, msg); 
  }

  for (int i = 0; i < numberOfCircules; i++) {
      CircuitPlayground.setPixelColor(i, 255, 0, 0);
  }
}

// Check for find my dog
BLYNK_WRITE(V1) {
  Serial.println(100);
  if (param.asInt()) {
    isFindMyDog = true;
  }
}

// Listen for humidity changes
BLYNK_WRITE(V0){ 
  humidityLevel = param.asDouble();
}

void handleHumidity() {
  // normlize between 0 and 5
  int numberOfCircules = ((humidityLevel - 0) / (100 - 0)) * 5;

  for (int i = 5; i < 5 + numberOfCircules; i++) {
     CircuitPlayground.setPixelColor(i, 0, 0, 255);
  }
}

void handleFindMyDog() {
  if (isFindMyDog) {
    CircuitPlayground.playTone(440, 0);
  } else {
    CircuitPlayground.playTone(0, 0);
  }

  if (CircuitPlayground.leftButton() || CircuitPlayground.rightButton()) {
    Serial.println("FOUND");
    isFindMyDog = false;
  }
}

void loop() {

  handleTemerature();
  handleBarking();
  handleFindMyDog();
  handleHumidity();
}

